package com.muyu.marketing.domain;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import javax.validation.constraints.NotNull;

import java.io.Serializable;

import java.util.Date;
import java.util.List;
import java.util.function.Supplier;

import ch.qos.logback.core.joran.spi.RuleStore;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.marketing.domain.model.ActivityTeamInfoAddModel;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.resp.RuleInfo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

/**
* 商品拼团信息
* @TableName activity_team_info
*/
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("activity_team_info")
@ApiModel(value = "ProjectInfo", description = "拼团活动信息")
public class ActivityTeamInfo extends BaseEntity {

    /**
    * 主键
    */

    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty("主键")
    private Long id;
    /**
    * 拼团名称
    */

    @ApiModelProperty("拼团名称")
    private String name;
    /**
    * 商品ID
    */

    @ApiModelProperty("商品ID")
    private Long productId;
    /**
    * 商品活动图
    */

    @ApiModelProperty("商品活动图")

    private String productImage;
    /**
    * 活动简介
    */

    @ApiModelProperty("活动简介")

    private String introduction;
    /**
    * 单位
    */

    @ApiModelProperty("单位")

    private String unit;
    /**
    * 轮播图
    */

    @ApiModelProperty("轮播图")

    private String imageList;
    /**
    * 活动结束时间
    */

    @ApiModelProperty("活动结束时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    private Date endTime;
    /**
    * 活动排序
    */

    @ApiModelProperty("活动排序")
    private Integer sort;
    /**
    * 活动详情
    */

    @ApiModelProperty("活动详情")

    private String content;
    /**
    * 活动状态
    */

    @ApiModelProperty("活动状态")

    private String status;
    /**
    * 策略类型
    */

    @ApiModelProperty("策略类型")

    private String strategyType;
    /**
    * 策略ID
    */

    @ApiModelProperty("策略ID")
    private Long strategyId;


    /**
    * 创建人
    */

    @ApiModelProperty("创建人")

    private String createBy;
    /**
    * 创建时间
    */
    @ApiModelProperty("创建时间")
    private Date createTime;
    /**
    * 修改人
    */

    @ApiModelProperty("修改人")

    private String updateBy;
    /**
    * 修改时间
    */
    @ApiModelProperty("修改时间")
    private Date updateTime;


    /**
     * 添加构造器
     */
    public static ActivityTeamInfo saveBuildModel(ActivityTeamInfoSaveReq activityTeamInfoAddModel, Supplier<String> createBy){
        return   ActivityTeamInfo.builder()
                .name(activityTeamInfoAddModel.getName())
                .productId(activityTeamInfoAddModel.getProductId())
                .productImage(activityTeamInfoAddModel.getProductImage())
                .introduction(activityTeamInfoAddModel.getIntroduction())
                .unit(activityTeamInfoAddModel.getUnit())
                .imageList(activityTeamInfoAddModel.getImageList() )
                .endTime(activityTeamInfoAddModel.getEndTime())
                .sort(activityTeamInfoAddModel.getSort())
                .content(activityTeamInfoAddModel.getContent())
                .strategyType(activityTeamInfoAddModel.getStrategyType())
                .strategyId(activityTeamInfoAddModel.getStrategyId())
                .content(activityTeamInfoAddModel.getContent())
                .status(activityTeamInfoAddModel.getStatus())
                .createBy(createBy.get())
                .createTime(new Date())
                .build();
    }


}
