package com.muyu.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.resp.RuleInfo;
import com.muyu.marketing.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.service.ActivityOpenInfoService;
import com.muyu.marketing.service.ActivityTeamInfoService;
import com.muyu.marketing.service.ActivityTeamProductSkuInfoService;
import com.muyu.product.domain.ProjectInfo;
import com.muyu.product.domain.model.ProjectAddModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper,ActivityTeamInfo> implements ActivityTeamInfoService {

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    @Autowired
    private ActivityOpenInfoService activityOpenInfoService;

    /**
     * 拼团活动列表
     * @param activityTeamInfoListQueryModel
     * @return
     */
    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query (ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()),ActivityTeamInfo::getName , activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()),ActivityTeamInfo::getStatus , activityTeamInfoListQueryModel.getStatus());
        Page<ActivityTeamInfo>  activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(),queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                                (activityTeamInfoListModelBuilder) -> {
                          Long openNumberByTeamIdAndType = activityOpenInfoService.getTeamOpenNumberByTeamIdAndType(activityTeamInfo.getId());
                          Long inNumberByTeamIdAndType = activityOpenInfoService.getTeamInNumberByTeamIdAndType(activityTeamInfo.getId());
//                          TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                          TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());

                                return  activityTeamInfoListModelBuilder
                                        .openTeamNumber(openNumberByTeamIdAndType)
                                        .addTeamNumber(inNumberByTeamIdAndType)
                                        .attendNumber(openNumberByTeamIdAndType + inNumberByTeamIdAndType)
                                        .teamPrice(BigDecimal.valueOf(1))
                                        .productPrice(BigDecimal.valueOf(1))
                                        .teamStock(teamProductStockModel.getTeamStock())
                                        .remainStock(teamProductStockModel.getRemainStock())
                                         .build();

                        })).toList();
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);

        return tableDataInfo;

    }

    /**
     * 拼团活动添加
     * @param teamInfoSaveReq
     * @return
     */
    @Transactional
    @Override
    public Result add(ActivityTeamInfoSaveReq teamInfoSaveReq) {
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.saveBuildModel(teamInfoSaveReq, SecurityUtils::getUsername);
        this.save(activityTeamInfo);
        List<RuleInfo> ruleInfoList = teamInfoSaveReq.getRuleInfo();
        List<ActivityTeamProductSkuInfo> activityTeamProductSkuInfos = ruleInfoList.stream().map(ruleInfo -> {
            ActivityTeamProductSkuInfo productSkuInfo = ActivityTeamProductSkuInfo.builder()
                    .teamId(activityTeamInfo.getId())
                    .productSku(ruleInfo.getSku())
                    .teamStock(ruleInfo.getStock())
                    .teamPrice(ruleInfo.getPrice())
                    .productId(teamInfoSaveReq.getProductId())
                    .createTime(new Date())
                    .build();
            return productSkuInfo;
        }).toList();
        activityTeamProductSkuInfoService.saveBatch(activityTeamProductSkuInfos);
        return Result.success(0,"添加成功");
    }
}
