package com.muyu.product.domain;

import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.annotation.Excel;
import com.muyu.product.domain.req.GroupBuyQueryReq;
import com.muyu.product.domain.req.GroupBuySaveReq;
import com.muyu.product.domain.req.GroupBuyEditReq;
import com.muyu.common.core.web.domain.BaseEntity;

/**
 * 拼团活动对象 group_buy
 *
 * @author DongZeLiang
 * @date 2024-11-14
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("group_buy")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "GroupBuy", description = "拼团活动")
public class GroupBuy extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** 编号 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "编号", value = "编号")
    private Long id;

    /** 外键：关联商品 */
    @Excel(name = "外键：关联商品")
    @ApiModelProperty(name = "外键：关联商品", value = "外键：关联商品")
    private Long productId;

    /** 开始时间 */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "开始时间", width = 30, dateFormat = "yyyy-MM-dd")
    @ApiModelProperty(name = "开始时间", value = "开始时间")
    private Date startTime;

    /** 结束时间 */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "结束时间", width = 30, dateFormat = "yyyy-MM-dd")
    @ApiModelProperty(name = "结束时间", value = "结束时间")
    private Date endTime;

    /** 最小成团人数 */
    @Excel(name = "最小成团人数")
    @ApiModelProperty(name = "最小成团人数", value = "最小成团人数")
    private Long minMembers;

    /** 活动转态 */
    @Excel(name = "活动转态")
    @ApiModelProperty(name = "活动转态", value = "活动转态")
    private Long status;

    /**
    * 查询构造器
    */
    public static GroupBuy queryBuild( GroupBuyQueryReq groupBuyQueryReq){
        return GroupBuy.builder()
                .productId(groupBuyQueryReq.getProductId())
                .startTime(groupBuyQueryReq.getStartTime())
                .endTime(groupBuyQueryReq.getEndTime())
                .minMembers(groupBuyQueryReq.getMinMembers())
                .status(groupBuyQueryReq.getStatus())
                .build();
    }

    /**
    * 添加构造器
    */
    public static GroupBuy saveBuild(GroupBuySaveReq groupBuySaveReq){
        return GroupBuy.builder()
                .productId(groupBuySaveReq.getProductId())
                .startTime(groupBuySaveReq.getStartTime())
                .endTime(groupBuySaveReq.getEndTime())
                .minMembers(groupBuySaveReq.getMinMembers())
                .status(groupBuySaveReq.getStatus())
                .build();
    }

    /**
    * 修改构造器
    */
    public static GroupBuy editBuild(Long id, GroupBuyEditReq groupBuyEditReq){
        return GroupBuy.builder()
            .id(id)
                .productId(groupBuyEditReq.getProductId())
                .startTime(groupBuyEditReq.getStartTime())
                .endTime(groupBuyEditReq.getEndTime())
                .minMembers(groupBuyEditReq.getMinMembers())
                .status(groupBuyEditReq.getStatus())
                .build();
    }

}
