package com.muyu.product.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.annotation.Excel;
import com.muyu.product.domain.req.GroupOrderQueryReq;
import com.muyu.product.domain.req.GroupOrderSaveReq;
import com.muyu.product.domain.req.GroupOrderEditReq;
import com.muyu.common.core.web.domain.BaseEntity;

/**
 * 拼团订单对象 group_order
 *
 * @author DongZeLiang
 * @date 2024-11-14
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("group_order")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "GroupOrder", description = "拼团订单")
public class GroupOrder extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /** 主键 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "主键", value = "主键")
    private Long id;

    /** 外键：关联拼团活动 */
    @Excel(name = "外键：关联拼团活动")
    @ApiModelProperty(name = "外键：关联拼团活动", value = "外键：关联拼团活动")
    private Long groupBuyId;

    /** 用户ID */
    @Excel(name = "用户ID")
    @ApiModelProperty(name = "用户ID", value = "用户ID")
    private Long userId;

    /** 订单状态 */
    @Excel(name = "订单状态")
    @ApiModelProperty(name = "订单状态", value = "订单状态")
    private Long status;

    /**
    * 查询构造器
    */
    public static GroupOrder queryBuild( GroupOrderQueryReq groupOrderQueryReq){
        return GroupOrder.builder()
                .groupBuyId(groupOrderQueryReq.getGroupBuyId())
                .userId(groupOrderQueryReq.getUserId())
                .status(groupOrderQueryReq.getStatus())
                .build();
    }

    /**
    * 添加构造器
    */
    public static GroupOrder saveBuild(GroupOrderSaveReq groupOrderSaveReq){
        return GroupOrder.builder()
                .groupBuyId(groupOrderSaveReq.getGroupBuyId())
                .userId(groupOrderSaveReq.getUserId())
                .status(groupOrderSaveReq.getStatus())
                .build();
    }

    /**
    * 修改构造器
    */
    public static GroupOrder editBuild(Long id, GroupOrderEditReq groupOrderEditReq){
        return GroupOrder.builder()
            .id(id)
                .groupBuyId(groupOrderEditReq.getGroupBuyId())
                .userId(groupOrderEditReq.getUserId())
                .status(groupOrderEditReq.getStatus())
                .build();
    }

}
