package com.muyu.product.domain.req;

import com.muyu.product.domain.model.CategoryInfoSaveModel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;
import com.muyu.common.core.web.domain.TreeEntity;

import java.util.Date;
import java.util.List;
import java.util.function.Supplier;

/**
 * 品类信息对象 category_info
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@ApiModel(value = "CategoryInfoEditReq", description = "品类信息")
public class CategoryInfoEditReq extends TreeEntity {

    private static final long serialVersionUID = 1L;

    /** 品类名称 */
    @ApiModelProperty(name = "品类名称", value = "品类名称", required = true)
    private String name;

    /** 图片 */
    @ApiModelProperty(name = "图片", value = "图片", required = true)
    private String image;

    /** 是否启用 */
    @ApiModelProperty(name = "是否启用", value = "是否启用", required = true)
    private String start;

    /** 介绍 */
    @ApiModelProperty(name = "介绍", value = "介绍")
    private String introduction;



    /**
     * 商品属性组关联ID
     */
    private List<Long> attributeGroupIdList;
    /**
     * 商品属性关联ID
     */
    private List<Long> attributeIdList;

    /**
     * 商品品牌组关联ID
     */
    private List<Long> brandIdList;

    /**
     * 添加构造器
     */
    public static CategoryInfoSaveModel saveBuild(CategoryInfoSaveReq categoryInfoSaveReq, Supplier<String> supplier){
        return CategoryInfoSaveModel.builder()
                .name(categoryInfoSaveReq.getName())
                .image(categoryInfoSaveReq.getImage())
                .start(categoryInfoSaveReq.getStart())
                .introduction(categoryInfoSaveReq.getIntroduction())
                .parentId(categoryInfoSaveReq.getParentId())
                .attributeGroupIdList(categoryInfoSaveReq.getAttributeGroupIdList())
                .attributeIdList(categoryInfoSaveReq.getAttributeIdList())
                .brandIdList(categoryInfoSaveReq.getBrandIdList())
                .createBy(supplier.get())
                .createTime(new Date())
                .build();
    }

}
