package com.muyu.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.product.domain.AsAttributeGroup;
import com.muyu.product.domain.AttributeGroup;
import com.muyu.product.domain.AttributeInfo;
import com.muyu.product.domain.model.AttributeGroupSaveModel;
import com.muyu.product.domain.req.AttributeGroupEditReq;
import com.muyu.product.domain.resp.AddAttributeGroupResp;
import com.muyu.product.domain.resp.AttributeGroupPageResp;
import com.muyu.product.domain.resp.AttributeGroupUpdResp;
import com.muyu.product.domain.resp.FindByIdResp;
import com.muyu.product.mapper.AsAttributeGroupMapper;
import com.muyu.product.mapper.AttributeGroupMapper;
import com.muyu.product.service.AsAttributeGroupService;
import com.muyu.product.service.AttributeGroupService;
import com.muyu.product.service.AttributeInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Stream;

/**
 * 属性组Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class AttributeGroupServiceImpl extends ServiceImpl<AttributeGroupMapper, AttributeGroup> implements AttributeGroupService {

    @Autowired
    private AsAttributeGroupService asAttributeGroupService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    @Autowired
    private AsAttributeGroupMapper asAttributeGroupMapper;

    /**
     * 分页查询
     *
     * @param attributeGroupQuery 分组对象
     *
     * @return 返回结果
     */
    @Override
    public TableDataInfo<AttributeGroupPageResp> page (AttributeGroup attributeGroupQuery) {
        List<AttributeGroup> list = this.list(attributeGroupQuery);
        List<AttributeGroupPageResp> pageRespList = list.stream()
                .map(attributeGroup ->
                        AttributeGroupPageResp.groupFunBuild(
                                attributeGroup, groupId -> attributeInfoService.attributeListByGroupId(groupId)
                        )
                )
                .toList();
        return TableDataInfo.<AttributeGroupPageResp>builder()
                .total(new PageInfo(list).getTotal())
                .rows(pageRespList)
                .build();
    }

    /**
     * 查询属性组列表
     *
     * @param attributeGroup 属性组
     *
     * @return 属性组
     */
    @Override
    public List<AttributeGroup> list (AttributeGroup attributeGroup) {
        LambdaQueryWrapper<AttributeGroup> queryWrapper = new LambdaQueryWrapper<>();

        if (ObjUtils.notNull(attributeGroup.getName())) {
            queryWrapper.like(AttributeGroup::getName, attributeGroup.getName());
        }

        if (ObjUtils.notNull(attributeGroup.getStates())) {
            queryWrapper.eq(AttributeGroup::getStates, attributeGroup.getStates());
        }

        return list(queryWrapper);
    }

    /**
     * 保存
     *
     * @param attributeGroupSaveModel 属性组保存模型
     *
     * @return 是否成功
     */
    @Override
    @Transactional
    public Boolean save (AttributeGroupSaveModel attributeGroupSaveModel) {
        AttributeGroup attributeGroup = attributeGroupSaveModel.buildAttributeGroup();
        boolean save = this.save(attributeGroup);
        // 处理属性ID
        Long attributeGroupId = attributeGroup.getId();
        List<Long> attributeIdList = attributeGroupSaveModel.getAttributeIdList();

        asAttributeGroupService.saveBatch(
                attributeIdList.stream()
                        .map(attributeId -> AsAttributeGroup.buildGroup(attributeGroupId, attributeId))
                        .toList()
        );
        return save;
    }

    @Override
    public FindByIdResp findById(Integer id) {
        return asAttributeGroupMapper.findById(id);
    }


    /**
     * 修改规格属性
     * @param
     * @return
     */
//    @Transactional
//    @Override
//    public Result upd(AddAttributeGroupResp addAttributeGroupResp) {
////        asAttributeGroupMapper.del(addAttributeGroupResp.getGroupId());
////        asAttributeGroupMapper.add(addAttributeGroupResp.getGroupId(),addAttributeGroupResp.getList());
//        return Result.success(0,"修改成功");
//    }

    /**
     * 商品规格属性回显
     * @param id
     * @return
     */
    @Override
    public AttributeGroupUpdResp getUpdById(Long id) {
        //查询编号
        AttributeGroup attributeGroup = this.getById(id);
        LambdaQueryWrapper<AsAttributeGroup> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsAttributeGroup::getGroupId, id);
        List<AsAttributeGroup> asAttributeGroups = asAttributeGroupService.list(queryWrapper);
        List<Long> attributeList = asAttributeGroups.stream()
                .map(AsAttributeGroup::getAttributeId)
                .toList();
        List<AttributeInfo> attributeInfoList = attributeInfoService.listByIds(attributeList);

        return AttributeGroupUpdResp.builder()
                .id(attributeGroup.getId())
                .name(attributeGroup.getName())
                .states(attributeGroup.getStates())
                .attributeList(attributeInfoList).build();


    }

    @Transactional
    @Override
    public Result updateAttibuteGroup(Long id, AttributeGroupEditReq attributeGroupEditReq) {
        asAttributeGroupMapper.del(id);
        asAttributeGroupMapper.add(id,attributeGroupEditReq.getAttributeList());
        return Result.success(0,"修改成功");
    }

    /**
     * 修改状态
     * @param states
     * @return
     */
    @Override
    public Result updateStatus(String states,Long id) {
        asAttributeGroupMapper.updStatus(states,id);
        return Result.success(0,"修改成功");
    }
}
