package com.muyu.product.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Stream;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.model.*;
import com.muyu.product.domain.req.ProjectInfoEditReq;
import com.muyu.product.domain.req.ProjectInfoSaveReq;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.ProjectDetailResp;
import com.muyu.product.domain.resp.ProjectInfoListResp;
import com.muyu.product.mapper.RuleInfoMapper;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.ProjectInfoMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.util.CollectionUtils;

/**
 * 商品信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class ProjectInfoServiceImpl extends ServiceImpl<ProjectInfoMapper, ProjectInfo>  implements ProjectInfoService {


    @Autowired
    private AsProductAttributeInfoService asProductAttributeInfoService;

    @Autowired
    private ProjectSkuInfoService projectSkuInfoService;

    @Autowired
    private CategoryInfoService categoryInfoService;

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    @Autowired
    private ProjectInfoMapper projectInfoMapper;

    @Autowired
    private RuleInfoMapper ruleInfoMapper;
    /**
     * 查询商品信息列表
     *
     * @param projectInfo 商品信息
     * @return 商品信息
     */
    @Override
    public List<ProjectInfo> list(ProjectInfo projectInfo) {
        LambdaQueryWrapper<ProjectInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(projectInfo.getName())){
            queryWrapper.like(ProjectInfo::getName, projectInfo.getName());
        }

        if (ObjUtils.notNull(projectInfo.getIntroduction())){
            queryWrapper.eq(ProjectInfo::getIntroduction, projectInfo.getIntroduction());
        }

        if (ObjUtils.notNull(projectInfo.getMianType())){
            queryWrapper.eq(ProjectInfo::getMianType, projectInfo.getMianType());
        }

        if (ObjUtils.notNull(projectInfo.getParentType())){
            queryWrapper.eq(ProjectInfo::getParentType, projectInfo.getParentType());
        }

        if (ObjUtils.notNull(projectInfo.getType())){
            queryWrapper.eq(ProjectInfo::getType, projectInfo.getType());
        }

        if (ObjUtils.notNull(projectInfo.getImage())){
            queryWrapper.eq(ProjectInfo::getImage, projectInfo.getImage());
        }

        if (ObjUtils.notNull(projectInfo.getCarouselImages())){
            queryWrapper.eq(ProjectInfo::getCarouselImages, projectInfo.getCarouselImages());
        }

        if (ObjUtils.notNull(projectInfo.getStatus())){
            queryWrapper.eq(ProjectInfo::getStatus, projectInfo.getStatus());
        }

        if (ObjUtils.notNull(projectInfo.getRuleId())){
            queryWrapper.eq(ProjectInfo::getRuleId, projectInfo.getRuleId());
        }

        if (ObjUtils.notNull(projectInfo.getBrandId())){
            queryWrapper.eq(ProjectInfo::getBrandId, projectInfo.getBrandId());
        }





        return list(queryWrapper);
    }

    /**
     * 保存商品信息
     *
     * @param projectInfoSaveReq 请求对象
     *
     * @return
     */
    @Override
    public boolean save (ProjectInfoSaveReq projectInfoSaveReq) {
        ProjectAddModel projectAddModel = projectInfoSaveReq.getProjectAddModel();
        ProjectInfo projectInfo = ProjectInfo.saveModelBuild(projectAddModel, SecurityUtils::getUsername);
        boolean save = this.save(projectInfo);
        if (save){
            // 属性值
            List<AttrValueModel> attrValueList = projectInfoSaveReq.getAttrValueList();
            List<AsProductAttributeInfo> asProductAttributeInfoList = attrValueList.stream()
                    .map(attrValueModel -> AsProductAttributeInfo.attrValueModelBuild(attrValueModel, projectInfo::getId))
                    .toList();
            if (!asProductAttributeInfoList.isEmpty()){
                asProductAttributeInfoService.saveBatch(asProductAttributeInfoList);
            }
            // sku ProductSkuModel -》 ProjectSkuInfo
            List<ProductSkuModel> productSkuModelList = projectInfoSaveReq.getProductSkuList();
            List<ProjectSkuInfo> projectSkuInfoList = productSkuModelList.stream().map(
                    productSkuModel -> ProjectSkuInfo.productSkuModelBuild(productSkuModel, projectInfo::getId)
            ).toList();
            if (!projectSkuInfoList.isEmpty()){
                projectSkuInfoService.saveBatch(projectSkuInfoList);
            }
        }
        return save;
    }

    @Override
    public Result add(ProjectInfoSaveReq projectInfoSaveReq) {
        String name = projectInfoSaveReq.getProjectAddModel().getName();
        LambdaQueryWrapper<ProjectInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ProjectInfo::getName,name);
        List<ProjectInfo> projectInfos = projectInfoMapper.selectList(queryWrapper);
        if(projectInfos.size()>0){
           return Result.success(500,"姓名重复，禁止添加");
        }

        save(projectInfoSaveReq);
        return Result.success(0,"添加成功");

    }


    /**
     * 通过商品ID获取商品详情
     *
     * @param id 商品ID
     * @return 商品详情
     */
    @Override
    public ProjectDetailResp getDetailInfo (Long id) {
        // 商品信息获取
        ProjectInfo projectInfo = this.getById(id);
        // 品牌信息
        BrandInfo brandInfo = this.brandInfoService.getById(projectInfo.getBrandId());
        // 品类集合
        List<CategoryInfo> categoryInfoList = categoryInfoService.listByIds(new ArrayList<>() {{
            add(projectInfo.getMianType());
            add(projectInfo.getParentType());
            add(projectInfo.getType());
        }});
        // 商品Sku集合
        List<ProjectSkuInfo> projectSkuInfoList = this.projectSkuInfoService.list(new LambdaQueryWrapper<>() {{
            eq(ProjectSkuInfo::getProjectId, id);
        }});

        // 商品和属性集合
        List<AsProductAttributeInfo> productAttributeInfoList = this.asProductAttributeInfoService.list(new LambdaQueryWrapper<>() {{
            eq(AsProductAttributeInfo::getProductId, id);
        }});
        // 商品规格
        List<RuleAttrAddModel> ruleAttrModelList = ruleAttrInfoService.list(new LambdaQueryWrapper<>() {{
                    eq(RuleAttrInfo::getRuleId, projectInfo.getRuleId());
                }}).stream()
                .map(RuleAttrAddModel::infoBuild).toList();


        CategoryCommonElementResp templateAttribute = this.categoryInfoService.getTemplateAttributeByCateGoryId(projectInfo.getType());
        List<TemplateAttributeGroupModel> templateAttributeGroupList = templateAttribute.getTemplateAttributeGroupList();
        List<TemplateAttributeModel> templateAttributeList = new ArrayList<>(){{
            addAll(templateAttribute.getTemplateAttributeList());
        }};
        // 属性组和商品属性的ID
        List<Long> notInAttributeIdList = new ArrayList<>();
        List<Long> attributeGroupIdList = templateAttributeGroupList.stream()
                .flatMap(templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                .map(TemplateAttributeModel::getId)
                .toList();
        List<Long> attributeIdList = templateAttributeList.stream()
                .map(TemplateAttributeModel::getId)
                .toList();
        if (!attributeGroupIdList.isEmpty()){
            notInAttributeIdList.addAll( attributeGroupIdList );
        }
        // 添加上，商品的自有属性
        List<AsProductAttributeInfo> productAttributeList = this.asProductAttributeInfoService.list(
                new LambdaQueryWrapper<>() {{
                    eq(AsProductAttributeInfo::getProductId, projectInfo.getId());
                    notIn(AsProductAttributeInfo::getAttributeId, notInAttributeIdList);
                }}
        );

        List<TemplateAttributeModel> projectAttributeList = new ArrayList<>();
        if (!productAttributeList.isEmpty()){
            List<Long> attrIdList = productAttributeList.stream()
                    .map(AsProductAttributeInfo::getAttributeId)
                    .toList();
            projectAttributeList = attributeInfoService.list(
                                            new LambdaQueryWrapper<>() {{
                                                in(AttributeInfo::getId, attrIdList);
                                            }}
                                        ).stream()
                                        .map(TemplateAttributeModel::attributeInfoBuild)
                                        .toList();
        }
        // 把自有属性添加到商品属性的集合当中，进行合并
        if (!projectAttributeList.isEmpty()){
            templateAttributeList.addAll(projectAttributeList);
        }
        return ProjectDetailResp.builder()
                .projectInfo(projectInfo)
                .brandInfo(brandInfo)
                .categoryInfoList(categoryInfoList)
                .projectSkuInfoList(projectSkuInfoList)
                .productAttributeInfoList(productAttributeInfoList)
                .ruleAttrModelList(ruleAttrModelList)
                .attributeInfoList(templateAttributeList)
                .attributeGroupList(templateAttributeGroupList)
                .build();
    }

    @Override
    public Result updAll(Long id, ProjectInfoEditReq projectInfoEditReq) {

        String name = projectInfoEditReq.getProjectAddModel().getName();
        LambdaQueryWrapper<ProjectInfo> objectLambdaQueryWrapper = new LambdaQueryWrapper<>();
        objectLambdaQueryWrapper.eq(ProjectInfo::getName,name);
        List<ProjectInfo> projectInfos = projectInfoMapper.selectList(objectLambdaQueryWrapper);
        if(projectInfos.size()>0){
            return Result.success(500,"姓名重复，禁止修改");
        }else {
            this.updateById(ProjectAddModel.editBuild(id, projectInfoEditReq.getProjectAddModel()));
            if (!CollectionUtils.isEmpty(projectInfoEditReq.getAttrValueList())) {
                LambdaQueryWrapper<AttributeInfo> queryWrapper = new LambdaQueryWrapper<AttributeInfo>().eq(AttributeInfo::getId, id);

                List<AttributeInfo> attributeInfos = projectInfoEditReq.getAttrValueList().stream().filter(attributeInfoUpdModel -> !StringUtils.isBlank(attributeInfoUpdModel.getName()))
                        .map(itm -> {
                            AttributeInfo attributeInfo = new AttributeInfo();
                            attributeInfo.setId(itm.getId());
                            attributeInfo.setName(itm.getName());

                            return attributeInfo;
                        }).toList();
                attributeInfoService.updateBatchById(attributeInfos);
            }

            if (!CollectionUtils.isEmpty(projectInfoEditReq.getProductSkuList())) {
                List<ProjectSkuInfo> skuInfoList = projectInfoEditReq.getProductSkuList().stream().map(proProductSku -> {
                    ProjectSkuInfo skuInfo = new ProjectSkuInfo();
                    skuInfo.setSku(proProductSku.getSku());
                    skuInfo.setImage(proProductSku.getImage());
                    skuInfo.setStock(proProductSku.getStock());
                    skuInfo.setProjectId(proProductSku.getProjectId());
                    return proProductSku;
                }).toList();

                skuInfoList.forEach(projectSkuInfo -> {
                    LambdaUpdateWrapper<ProjectSkuInfo> updateWrapper = new LambdaUpdateWrapper<>();
                    if (!StringUtils.isEmpty(projectSkuInfo.getImage())) {

                    }
                    updateWrapper.set(ProjectSkuInfo::getStock, projectSkuInfo.getStock());
                    updateWrapper.set(ProjectSkuInfo::getPrice, projectSkuInfo.getPrice());
                    updateWrapper.eq(ProjectSkuInfo::getSku, projectSkuInfo.getSku());
                    projectSkuInfoService.update(null, updateWrapper);
                });
            }
            return Result.success(0, "修改成功");
        }
    }

    @Override
    public List<ProjectInfoListResp> pageInfo(ProjectInfo projectInfo) {
        List<ProjectInfo> list = list(projectInfo);
        LinkedList<ProjectInfoListResp> objectLinkedList = new LinkedList<>();
        list.forEach( n ->{
            CategoryInfo mianTypeName = categoryInfoService.getById(n.getRuleId());
            CategoryInfo byIdParent = categoryInfoService.getById(n.getParentType());
          CategoryInfo typeNum =  n.getType()!=null ? categoryInfoService.getById(n.getType()) :null;

            RuleInfo ruleName = ruleInfoMapper.selectById(n.getRuleId());
            BrandInfo brandName = brandInfoService.getById(n.getBrandId());
            ProjectInfoListResp build = ProjectInfoListResp.builder()
                    .id(n.getId())
                    .name(n.getName())
                    .introduction(n.getIntroduction())
                    .type(n.getType())
                    .parentType(n.getParentType())
                    .type(n.getType())
                    .image(n.getImage())
                    .carouselImages(n.getCarouselImages())
                    .status(n.getStatus())
                    .mianTypeName(mianTypeName != null ? brandName.getNam() :"")
                    .parentTypeName(byIdParent !=null ? byIdParent.getName(): "")
                    .typeName(typeNum != null ? typeNum.getName() : "")
                    .ruleName(ruleName != null ? ruleName.getName() :"")
                    .brandName(brandName != null ? brandName.getNam():"")
                    .build();
            objectLinkedList.add(build);
        });

        return objectLinkedList;

    }
}
