package com.muyu.product.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.text.Convert;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.model.RuleAttrAddModel;
import com.muyu.product.domain.model.RuleInfoAddModel;
import com.muyu.product.domain.req.RuleInfoEditReq;
import com.muyu.product.domain.req.RuleInfoQueryReq;
import com.muyu.product.domain.resp.AttributeGroupUpdResp;
import com.muyu.product.domain.resp.RuleInfoResp;
import com.muyu.product.mapper.RuleAttrInfoMapper;
import com.muyu.product.service.RuleAttrInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.RuleInfoMapper;
import com.muyu.product.service.RuleInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.transaction.annotation.Transactional;

/**
 * 商品规格Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class RuleInfoServiceImpl extends ServiceImpl<RuleInfoMapper, RuleInfo>  implements RuleInfoService {

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;

    @Autowired
    private RuleAttrInfoMapper ruleAttrInfoMapper;

    @Autowired
    private RuleInfoMapper ruleInfoMapper;

    /**
     * 查询商品规格列表
     *
     * @param ruleInfo 商品规格
     * @return 商品规格
     */
    @Override
    public List<RuleInfo> list(RuleInfo ruleInfo) {
        LambdaQueryWrapper<RuleInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleInfo.getName())){
            queryWrapper.like(RuleInfo::getName, ruleInfo.getName());
        }

        if (ObjUtils.notNull(ruleInfo.getStatus())){
            queryWrapper.eq(RuleInfo::getStatus, ruleInfo.getStatus());
        }





        return list(queryWrapper);
    }

    /**
     * 添加规格信息
     *
     * @param ruleInfoAddModel 规格添加模型
     *
     * @return 时候成功
     */
    @Override
    public boolean save (RuleInfoAddModel ruleInfoAddModel) {
        RuleInfo ruleInfo = RuleInfo.addModelBuild(ruleInfoAddModel, SecurityUtils::getUsername);
        boolean save = this.save(ruleInfo);
        ruleAttrInfoService.saveBatch(
                ruleInfoAddModel.getRuleAttrList().stream()
                        .map(ruleAttrAddModel -> RuleAttrInfo.addModelBuild(ruleAttrAddModel, ruleInfo::getId, SecurityUtils::getUsername))
                        .toList()
        );
        return save;
    }

    /**
     * 查询规格信息
     * @param ruleInfoQueryReq 规格查询
     * @return 规格信息
     */
    @Override
    public TableDataInfo<RuleInfoResp> queryList (RuleInfoQueryReq ruleInfoQueryReq) {


        List<RuleInfo> list = this.list(RuleInfo.queryBuild(ruleInfoQueryReq));
        List<RuleInfoResp> ruleInfoRespList = list.stream()
                .map(ruleInfo -> RuleInfoResp.infoBuild(ruleInfo, ruleId -> {
                        LambdaQueryWrapper<RuleAttrInfo> queryWrapper = new LambdaQueryWrapper<>();
                        queryWrapper.eq(RuleAttrInfo::getRuleId, ruleId);
                        return ruleAttrInfoService.list(queryWrapper).stream().map(RuleAttrAddModel::infoBuild).toList();
                    }))
                .toList();
        boolean isPage = ruleInfoQueryReq.getParams().get("isPage") == null || Convert.toBool(ruleInfoQueryReq.getParams().get("isPage"), true);

        return TableDataInfo.<RuleInfoResp>builder()
                .rows(ruleInfoRespList)
                .total(isPage ? new PageInfo<>(list).getTotal() : 0)
                .build();
    }

    @Override
    public Result<RuleInfoResp> getRuleById(Long id) {
      RuleInfoResp getRuleById  = ruleAttrInfoMapper.getRuleById(id);
        List<RuleAttrAddModel> ruleAttrList = getRuleById.getRuleAttrList();
        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
            String attrValue = ruleAttrAddModel.getAttrValue();
            String[] split = attrValue.split(",");
           ruleAttrAddModel.setValueList(List.of(split));
        }
        return Result.success(getRuleById);
    }

    @Transactional
    @Override
    public Result upd(Long id, RuleInfoEditReq ruleInfoEditReq) {



        ruleAttrInfoMapper.del(id);
        List<RuleAttrAddModel> ruleAttrList = ruleInfoEditReq.getRuleAttrList();
        for (RuleAttrAddModel ruleAttrAddModel : ruleAttrList) {
            List<String> valueList = ruleAttrAddModel.getValueList();
            String join = String.join(",", valueList);
//            for (String s : valueList) {
//                ruleAttrInfoMapper.add(id,ruleAttrAddModel.getName(),s);
//            }
            ruleAttrInfoMapper.add(id,ruleAttrAddModel.getName(),join);
        }

        return Result.success(0,"修改成功");
    }

    @Override
    public Result updateStatus(String status, Long id) {
        LambdaUpdateWrapper<RuleInfo> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(RuleInfo::getId,id);
        updateWrapper.set(RuleInfo::getStatus,status);
        ruleInfoMapper.update(null,updateWrapper);
//        ruleAttrInfoMapper.updateStatus(status,id);
        return Result.success(0,"修改成功");
    }

    @Override
    public Result have(Long id) {
        Integer count =  ruleAttrInfoMapper.selectByAttributeGroupCount(id);
        if(count>0){

            return Result.error(400,"有商品关联禁止操作");
        }
        return Result.success(count);
    }
}
